@@ -4,25 +4,17 @@ package Data::Record::Serialize::Encode::json;
 
 use Moo::Role;
 
-our $VERSION = '0.13';
+our $VERSION = '0.15';
 
 use JSON::MaybeXS qw[ encode_json ];
 
 use namespace::clean;
 
-before BUILD => sub {
+has '+_numify'    => ( is => 'rwp', default => 1 );
+has '+_needs_eol' => ( is => 'rwp', default => 1 );
 
-    my $self = shift;
-
-    $self->_set__numify( 1 );
-    $self->_set__needs_eol( 1 );
-};
-
-#pod =begin pod_coverage
-#pod
-#pod =head3 encode
-#pod
-#pod =end pod_coverage
+#pod =for Pod::Coverage
+#pod   encode
 #pod
 #pod =cut
 
@@ -32,6 +24,18 @@ with 'Data::Record::Serialize::Role::Encode';
 
 1;
 
+#
+# This file is part of Data-Record-Serialize
+#
+# This software is Copyright (c) 2017 by Smithsonian Astrophysical Observatory.
+#
+# This is free software, licensed under:
+#
+#   The GNU General Public License, Version 3, June 2007
+#
+
+__END__
+
 =pod
 
 =head1 NAME
@@ -40,7 +44,7 @@ Data::Record::Serialize::Encode::json - encoded a record as JSON
 
 =head1 VERSION
 
-version 0.13
+version 0.15
 
 =head1 SYNOPSIS
 
@@ -64,18 +68,14 @@ read by an incremental decoder, e.g.
 
   @data = JSON->new->incr_parse( $json );
 
-It performs the L<B<Data::Record::Serialize::Role::Encode>> role.
+It performs the L<Data::Record::Serialize::Role::Encode> role.
 
-=begin pod_coverage
-
-=head3 encode
-
-=end pod_coverage
+=for Pod::Coverage encode
 
 =head1 INTERFACE
 
 There are no additional attributes which may be passed to
-L<B<Data::Record::Serialize-E<gt>new>|Data::Record::Serialize/new>.
+L<Data::Record::Serialize-E<gt>new>|Data::Record::Serialize/new>.
 
 =head1 BUGS AND LIMITATIONS
 
@@ -107,35 +107,3 @@ This is free software, licensed under:
   The GNU General Public License, Version 3, June 2007
 
 =cut
-
-__END__
-
-#pod =head1 SYNOPSIS
-#pod
-#pod     use Data::Record::Serialize;
-#pod
-#pod     my $s = Data::Record::Serialize->new( encode => 'json', ... );
-#pod
-#pod     $s->send( \%record );
-#pod
-#pod =head1 DESCRIPTION
-#pod
-#pod B<Data::Record::Serialize::Encode::json> encodes a record as JSON.
-#pod
-#pod If a field's type is C<N> or C<I>, it will be properly encoded by JSON
-#pod as a number.
-#pod
-#pod The output consists of I<concatenated> JSON objects, and is mostly easily
-#pod read by an incremental decoder, e.g.
-#pod
-#pod   use JSON::MaybeXS;
-#pod
-#pod   @data = JSON->new->incr_parse( $json );
-#pod
-#pod It performs the L<B<Data::Record::Serialize::Role::Encode>> role.
-#pod
-#pod
-#pod =head1 INTERFACE
-#pod
-#pod There are no additional attributes which may be passed to
-#pod L<B<Data::Record::Serialize-E<gt>new>|Data::Record::Serialize/new>.
