@@ -4,24 +4,18 @@ package Data::Record::Serialize::Encode::null;
 
 use Moo::Role;
 
-our $VERSION = '0.13';
+our $VERSION = '0.15';
 
 use namespace::clean;
 
+has '+_need_types' => ( is => 'rwp', default => 0 );
 
-#pod =begin pod_coverage
-#pod
-#pod =head3 encode
-#pod
-#pod =head3 send
-#pod
-#pod =head3 print
-#pod
-#pod =head3 say
-#pod
-#pod =head3 close
-#pod
-#pod =end pod_coverage
+#pod =for Pod::Coverage
+#pod  encode
+#pod  send
+#pod  print
+#pod  say
+#pod  close
 #pod
 #pod =cut
 
@@ -36,6 +30,18 @@ with 'Data::Record::Serialize::Role::Sink';
 
 1;
 
+#
+# This file is part of Data-Record-Serialize
+#
+# This software is Copyright (c) 2017 by Smithsonian Astrophysical Observatory.
+#
+# This is free software, licensed under:
+#
+#   The GNU General Public License, Version 3, June 2007
+#
+
+__END__
+
 =pod
 
 =head1 NAME
@@ -44,7 +50,7 @@ Data::Record::Serialize::Encode::null - infinite bitbucket
 
 =head1 VERSION
 
-version 0.13
+version 0.15
 
 =head1 SYNOPSIS
 
@@ -59,27 +65,19 @@ version 0.13
 B<Data::Record::Serialize::Encode::null> is both an encoder and a sink.
 All records sent using it will disappear.
 
-It performs both the L<B<Data::Record::Serialize::Role::Encode>> and
-L<B<Data::Record::Serialize::Role::Sink>> roles.
+It performs both the L<Data::Record::Serialize::Role::Encode> and
+L<Data::Record::Serialize::Role::Sink> roles.
 
-=begin pod_coverage
-
-=head3 encode
-
-=head3 send
-
-=head3 print
-
-=head3 say
-
-=head3 close
-
-=end pod_coverage
+=for Pod::Coverage encode
+ send
+ print
+ say
+ close
 
 =head1 INTERFACE
 
 There are no additional attributes which may be passed to
-L<B<Data::Record::Serialize::new>|Data::Record::Serialize/new>.
+L<Data::Record::Serialize::new>|Data::Record::Serialize/new>.
 
 =head1 BUGS AND LIMITATIONS
 
@@ -111,26 +109,3 @@ This is free software, licensed under:
   The GNU General Public License, Version 3, June 2007
 
 =cut
-
-__END__
-
-#pod =head1 SYNOPSIS
-#pod
-#pod     use Data::Record::Serialize;
-#pod
-#pod     my $s = Data::Record::Serialize->new( encode => 'null', ... );
-#pod
-#pod     $s->send( \%record );
-#pod
-#pod =head1 DESCRIPTION
-#pod
-#pod B<Data::Record::Serialize::Encode::null> is both an encoder and a sink.
-#pod All records sent using it will disappear.
-#pod
-#pod It performs both the L<B<Data::Record::Serialize::Role::Encode>> and
-#pod L<B<Data::Record::Serialize::Role::Sink>> roles.
-#pod
-#pod =head1 INTERFACE
-#pod
-#pod There are no additional attributes which may be passed to
-#pod L<B<Data::Record::Serialize::new>|Data::Record::Serialize/new>.
