@@ -5,7 +5,9 @@ package Data::Record::Serialize::Sink::stream;
 
 use Moo::Role;
 
-our $VERSION = '0.13';
+use Data::Record::Serialize::Error { errors => [ '::create' ] }, -all;
+
+our $VERSION = '0.15';
 
 use IO::File;
 
@@ -26,20 +28,15 @@ has fh => (
         return ( ! defined $self->output || $self->output eq '-' )
           ? \*STDOUT
           : ( IO::File->new( $self->output, 'w' )
-              or croak( "unable to create @{[ $self->output ]}\n" ) );
+              or error( '::create', "unable to create @{[ $self->output ]}" ) );
     },
 
 );
 
-#pod =begin pod_coverage
-#pod
-#pod =head3 print
-#pod
-#pod =head3 say
-#pod
-#pod =head3 close
-#pod
-#pod =end pod_coverage
+#pod =for Pod::Coverage
+#pod  print
+#pod  say
+#pod  close
 #pod
 #pod =cut
 
@@ -51,6 +48,18 @@ with 'Data::Record::Serialize::Role::Sink';
 
 1;
 
+#
+# This file is part of Data-Record-Serialize
+#
+# This software is Copyright (c) 2017 by Smithsonian Astrophysical Observatory.
+#
+# This is free software, licensed under:
+#
+#   The GNU General Public License, Version 3, June 2007
+#
+
+__END__
+
 =pod
 
 =head1 NAME
@@ -59,7 +68,7 @@ Data::Record::Serialize::Sink::stream - output encoded data to a stream.
 
 =head1 VERSION
 
-version 0.13
+version 0.15
 
 =head1 SYNOPSIS
 
@@ -74,22 +83,16 @@ version 0.13
 B<Data::Record::Serialize::Sink::stream> outputs encoded data to a
 file handle.
 
-It performs the L<B<Data::Record::Serialize::Role::Sink>> role.
+It performs the L<Data::Record::Serialize::Role::Sink> role.
 
-=begin pod_coverage
-
-=head3 print
-
-=head3 say
-
-=head3 close
-
-=end pod_coverage
+=for Pod::Coverage print
+ say
+ close
 
 =head1 INTERFACE
 
 The following attributes may be passed to
-L<B<Data::Record::Serialize-E<gt>new>|Data::Record::Serialize/new>:
+L<Data::Record::Serialize-E<gt>new>|Data::Record::Serialize/new>:
 
 =over
 
@@ -137,42 +140,3 @@ This is free software, licensed under:
   The GNU General Public License, Version 3, June 2007
 
 =cut
-
-__END__
-
-#pod =head1 SYNOPSIS
-#pod
-#pod     use Data::Record::Serialize;
-#pod
-#pod     my $s = Data::Record::Serialize->new( sink => 'stream', ... );
-#pod
-#pod     $s->send( \%record );
-#pod
-#pod =head1 DESCRIPTION
-#pod
-#pod B<Data::Record::Serialize::Sink::stream> outputs encoded data to a
-#pod file handle.
-#pod
-#pod It performs the L<B<Data::Record::Serialize::Role::Sink>> role.
-#pod
-#pod
-#pod =head1 INTERFACE
-#pod
-#pod The following attributes may be passed to
-#pod L<B<Data::Record::Serialize-E<gt>new>|Data::Record::Serialize/new>:
-#pod
-#pod =over
-#pod
-#pod =item C<output>
-#pod
-#pod The name of an output file or a reference to a scalar to which the records will be written.
-#pod C<output> may be set to C<-> to indicate output to the standard output stream.
-#pod
-#pod =item C<fh>
-#pod
-#pod A file handle.
-#pod
-#pod =back
-#pod
-#pod If neither is specified, output is written to the standard output
-#pod stream.
